@@ -30,6 +30,11 @@
 #include "luks.h"
 #include "config.h"
 
+/* Enable autodetection if possible */
+#ifdef HAVE_BLKID
+#include <blkid/blkid.h>
+#endif
+
 /* Using our private realpath function */
 #include "realpath.h"
 
@@ -57,7 +62,7 @@ usage( const char* exename )
     printf( _("Usage:\n\n%s [options] <device> [<label>]\n\n"
     "  Mount <device> to a directory below %s if policy requirements\n"
     "  are met (see pmount(1) for details). If <label> is given, the mount point\n"
-    "  will be %s/<label>, otherwise it will be %s<device>.\n"
+    "  will be %s<label>, otherwise it will be %s<device>.\n"
     "  If the mount point does not exist, it will be created.\n\n"),
         exename, MEDIADIR, MEDIADIR, MEDIADIR );
 
@@ -373,24 +378,53 @@ do_mount_auto( const char* device, const char* mntpt, int async,
     const struct FS* fs;
     int nostderr = 1;
     int result = -1;
+    struct stat buf;		/* Not used */
+
+    /* First, if that is supported, we try with blkid */
+#ifdef HAVE_BLKID
+    const char * tp;
+    blkid_cache c;
+    blkid_get_cache(&c, "/dev/null");
+    tp = blkid_get_tag_value(c, "TYPE", device);
+    if(tp) {
+      debug("blkid gave FS %s for '%s'\n", tp, device);
+      if(! strcmp(tp, "ntfs") && !stat(MOUNT_NTFS_3G, &buf)) {
+	debug("blkdid detected ntfs and ntfs-3g was found. Using ntfs-3g\n");
+	tp = "ntfs-3g";
+      }
+      result = do_mount( device, mntpt, tp, async, noatime, exec, 
+			 force_write, iocharset, umask, fmask, 
+			 dmask, nostderr );
+      if(result == 0)
+	return result;
+      debug("blkid-detected FS failed, trying manually \n");
+    }
+    blkid_put_cache(c);
+#endif /* HAVE_BLKID */
+
+    result = -1;
 
     for( fs = get_supported_fs(); fs->fsname; ++fs ) {
-        if(fs->skip_autodetect)
-	  continue;		/* skip fs that are marked as such */
-        /* don't suppress stderr if we try the last possible fs */
-        if( (fs+1)->fsname == NULL )
-            nostderr = 0;
-        result = do_mount( device, mntpt, fs->fsname, async, noatime, exec,
-                force_write, iocharset, umask, fmask, dmask, nostderr );
-        if( result == 0 )
-            break;
+      /* Skip fs marked as such unless it is ntfs-3g and
+	 we can stat MOUNT_NTFS_G3 
+      */
+      if(fs->skip_autodetect && 
+	 !(! strcmp(fs->fsname, "ntfs-3g") && !stat(MOUNT_NTFS_3G, &buf)))
+	continue;		/* skip fs that are marked as such */
+      /* don't suppress stderr if we try the last possible fs */
+      if( (fs+1)->fsname == NULL )
+	nostderr = 0;
+      result = do_mount( device, mntpt, fs->fsname, async, noatime, exec,
+			 force_write, iocharset, umask, fmask, dmask, nostderr );
+      if( result == 0 )
+	break;
 
-	/* sometimes VFAT fails when using iocharset; try again without */
-	if( iocharset )
-	    result = do_mount( device, mntpt, fs->fsname, async, noatime, exec,
-                    force_write, NULL, umask, fmask, dmask, nostderr );
-        if( result <= 0 )
-            break;
+      /* sometimes VFAT fails when using iocharset; try again without */
+      if( iocharset )
+	result = do_mount( device, mntpt, fs->fsname, async, noatime, exec,
+			   force_write, NULL, umask, fmask, dmask, nostderr );
+      if( result <= 0 )
+	break;
     }
     return result;
 }
@@ -669,8 +703,11 @@ main( int argc, char** argv )
         return E_ARGS;
     }
 
-    /* if we got a mount point, convert it to a device */
-    if( fstab_has_mntpt( "/etc/fstab", devarg, mntptdev, sizeof(mntptdev) ) ) {
+    /* Lookup in /etc/fstab if devarg is a mount point, unless we already
+       have a block device -- this way, pmount shouldn't choke on stale
+       network mounts. */
+    if( (! is_block(devarg)) && 
+	fstab_has_mntpt( "/etc/fstab", devarg, mntptdev, sizeof(mntptdev) ) ) {
         debug( "resolved mount point %s to device %s\n", devarg, mntptdev );
         devarg = mntptdev;
     }
@@ -707,6 +744,16 @@ main( int argc, char** argv )
                 return E_DEVICE;
             }
             debug( "trying to prepend '" DEVDIR "' to device argument, now %s\n", device );
+	    /* We need to lookup again in fstab: */
+	    fstab_device = fstab_has_device( "/etc/fstab", device, NULL, NULL );
+	    if( mode == MOUNT && fstab_device ) {
+	      if( arg2 )
+		fprintf( stderr, _("Warning: device %s is already handled by /etc/fstab,"
+				   " supplied label is ignored\n"), fstab_device );
+	      
+	      do_mount_fstab( fstab_device );
+	      return E_EXECMOUNT;
+	    }
         }
     }
 
