@@ -310,6 +310,8 @@ fstab_has_device( const char* fname, const char* device, char* mntpt, int *uid )
     char* realdev;
     char* uidopt;
 
+    debug("Checking for device '%s' in '%s'\n", device, fname);
+
     if( !( f = fopen( fname, "r" ) ) ) {
         perror( _("Error: could not open fstab-type file") );
         exit( 100 );
@@ -318,7 +320,12 @@ fstab_has_device( const char* fname, const char* device, char* mntpt, int *uid )
     while( ( entry = getmntent( f ) ) != NULL ) {
         snprintf( fstab_device, sizeof( fstab_device ), "%s", entry->mnt_fsname );
 
-        if( realpath( fstab_device, pathbuf ) )
+	/* 
+	   We deactivate the symlink lookup for fstab files: it does
+	   more harm than necessary. After all, the admin should know what
+	   they are doing when they are writing symlinks in /etc/fstab.
+	*/
+        if( 0 && realpath( fstab_device, pathbuf ) )
             realdev = pathbuf;
         else
             realdev = fstab_device;
